#![no_std]
#![no_main]

use core::panic::PanicInfo;
use core::sync::atomic::*;
use cty::c_void;

#[panic_handler]
fn panic(_info: &PanicInfo) -> ! {
    loop {}
}

#[no_mangle]
pub extern "C" fn stack_starving_callback() {}

extern "C" {
    fn printf(_: *const i8, ...);

    fn freertos_rs_spawn_task(
        f: extern "C" fn(_: *mut c_void),
        value: *mut c_void,
        name: *const u8,
        name_len: u8,
        stack_size: u16,
        priority: u32,
        task_handle: *mut c_void,
    ) -> u32;
}

static ABOOL: AtomicBool = AtomicBool::new(false);
static AUSIZE: AtomicUsize = AtomicUsize::new(0);
const ORDER: Ordering = Ordering::SeqCst;

#[no_mangle]
fn app_main() {
    spawn_task(0);
    spawn_task(1);
}

fn spawn_task(task_num: u32) {
    let task_name = "the task";
    let task_len = task_name.len() as u8;
    let task_name = task_name.as_ptr();
    let mut task_handle: u32 = 0;

    unsafe {
        printf("Starting task %d...\n\0".as_ptr() as *const _, task_num);
        freertos_rs_spawn_task(
            task,
            task_num as *mut _,
            task_name,
            task_len,
            2048,
            1,
            &mut task_handle as *mut u32 as *mut _,
        );
    };
}

extern "C" fn task(task_handle: *mut c_void) {
    unsafe {
        let task_num = task_handle as u32;

        loop {
            let abool = ABOOL.load(ORDER);
            let ausize = AUSIZE.load(ORDER);

            printf(
                "task %d: %d %d\n\0".as_ptr() as *const _,
                task_num,
                abool as u32,
                ausize,
            );

            ABOOL.swap(!abool, ORDER);
            AUSIZE.swap(ausize + 1, ORDER);
        }
    }
}
